/* Program09_1
 *
 * Ez a program az I2C1 buszon kiküld egy parancsot a 
 * DS32231 RTC-nek, hogy kapcsolja be az 1 Hz-es SQW kimenetet.
 * Az SQW kimenet nyitott nyelőelektródás, tehát "lefelé húz" 
 * (sink). Kössünk egy 1 k ellenállással sorbakötött LED-et
 * a tápfeszültség és az SQW kivezetés közé, hogy lássuk az
 * 1 Hz-es kimenet működését!
 *
 * Kapcsolás:
 *   --------           -------------
 *  | STM32  |         | DS3231 RTC  |  LED    1k
 *  |    PB9 |---------| SDA     SQW |--|<|--/\/\/\--+
 *  |    PB8 |---------| SCL         |               | 
 *  |     5V |---------| VCC     VCC |---------------+
 *  |    GND |---------| GND         |
 *   --------           --------------
 * Felhasznált forrás: 
 * Mazidi et al., STM32 Arm Programming for Embedded Systems    
 * http://www.microdigitaled.com/ARM/STM_ARM_books.html
 *
 * Tested with Keil MDK-Lite v5.28 and STM32F1 DFP v2.3.0
 */
 
#include "stm32f10x.h"
void I2C1_init(void);
int I2C1_byteWrite(char saddr, char maddr, char data);
#define SLAVE_ADDR 0x68    /* 1101 000.    DS3231 */

int main(void) {
    I2C1_init();
    I2C1_byteWrite(SLAVE_ADDR, 0x0E, 0);
    while(1) {
    }
}

void I2C1_init(void) {
  RCC->APB1ENR |=  RCC_APB1ENR_I2C1EN;	      // I2C1 engedélyezése
  RCC->APB2ENR |= RCC_APB2ENR_AFIOEN;         // AFIO engedélyezése	
  RCC->APB2ENR |=  RCC_APB2ENR_IOPBEN;        // GPIOB engedélyezése
  AFIO->MAPR |= AFIO_MAPR_I2C1_REMAP;         // I2C1 REMAP=1 (SCL/PB8, SDA/PB9)
  GPIOB->CRH |= GPIO_CRH_CNF8|GPIO_CRH_MODE8; // PB8 ALT/OD kimenet (SCL)
  GPIOB->CRH |= GPIO_CRH_CNF9|GPIO_CRH_MODE9; // PB9 ALT/OD kimenet (SDA) 
  I2C1->CR1 =   I2C_CR1_SWRST;                // Szoftver RESET
  I2C1->CR1 &= ~I2C_CR1_SWRST;                // RESET vége
	/* FREQ = PCLK1/1_000_000         */ 
  I2C1->CR2 = 36;                             // PCLK1 = 36 MHz
	/* CCR = max(PCLK1/ClkSpeed/2,4)  */
  I2C1->CCR = 180;                            // Standard mód 100 kHz
	/* TRISE = PCLK1/1000000+1        */
  I2C1->TRISE = 37;                           // maximum felfutási idő
  I2C1->CR1 |=  I2C_CR1_PE;                   // I2C1 modul indítása
}

/* Ez a függvény kiír egy adatbájtot (data) a saddr című 
 * I2C eszköz maddr című regiszterébe. Az egyszerűség kedvéért
 * nincs ellenőrzés, sem hibajelzés.
 */
int I2C1_byteWrite(char saddr, char maddr, char data) {
    volatile int tmp;
    
    while (I2C1->SR2 & 2);                  /* Vár, amíg a busz foglalt */
    
    I2C1->CR1 |=  0x100;                    /* START generálás */
    while (!(I2C1->SR1 & 1));               /* Vár, amíg SR1.SB = 0 */
    
    I2C1->DR = saddr << 1;                  /* Slave cím kiküldése */
    while (!(I2C1->SR1 & 2));               /* Vár, amíg SR1.ADDR = 0 */
    tmp = I2C1->SR2;                        /* SR1.ADDR törlése */
    
    while (!(I2C1->SR1 & 0x80));            /* Vár, amíg SR1.TXE = 0 */
    I2C1->DR = maddr;                       /* Memóriacím küldése */
    
    while (!(I2C1->SR1 & 0x80));            /* Vár, amíg SR1.TXE = 0 */
    I2C1->DR = data;                        /* Adat küldése */
    
    while (!(I2C1->SR1 & 4));               /* Vár, amíg SR1.BTF = 0  */
    I2C1->CR1 |= 0x200;                     /* DTOP generálás */
    return 0;
}







